UNIT Liste;

{$I OsWahl.inc}

{$IFNDEF DOS} {$MODE OBJFPC} {$H+} {$ENDIF}

INTERFACE

TYPE

  {$IFDEF DOS}

  PListeElement = ^TListeElement;
  TListeElement = RECORD
    Data: Pointer;
    Next: PListeElement;
  END;

  TListe = OBJECT
    Head, Tail: PListeElement;
    DataSize: Integer;
    Count: Integer;

  {$ELSE}

  GENERIC TListe<T> = OBJECT
    Public
      TYPE
        PListeElement = ^TListeElement;
        TListeElement = RECORD
          Data: T;
          Next: PListeElement;
        END;
      VAR
        Head, Tail: PListeElement;
        Count: Integer;

  {$ENDIF}

  Public
    PROCEDURE Init {$IFDEF DOS} (SizeOfData: Integer) {$ENDIF} ;
    PROCEDURE Add(Data: {$IFDEF DOS} Pointer {$ELSE} T {$ENDIF});
    FUNCTION Get(Index: Integer; {$IFDEF DOS} Data: Pointer {$ELSE} VAR Data: T {$ENDIF}): BOOLEAN;
    FUNCTION Remove(Index: Integer): BOOLEAN;
    FUNCTION IsEmpty: BOOLEAN;
    PROCEDURE Clear;
    FUNCTION Anzahl: Integer;
    FUNCTION Start: PListeElement;
  END;

IMPLEMENTATION

{$IFDEF DOS} USES DOS; {$ENDIF}

PROCEDURE TListe.Init {$IFDEF DOS} (SizeOfData: Integer) {$ENDIF} ;
BEGIN
  Head := NIL;
  Tail := NIL;
  Count := 0;
  {$IFDEF DOS} DataSize := SizeOfData; {$ENDIF}
END;

PROCEDURE TListe.Add(Data: {$IFDEF DOS} Pointer {$ELSE} T {$ENDIF});
VAR
  NewElement: PListeElement;
BEGIN
  New(NewElement);
  {$IFDEF DOS}
    GetMem(NewElement^.Data, DataSize);
    Move(Data^, NewElement^.Data^, DataSize);
  {$ELSE}
    NewElement^.Data := Data;
  {$ENDIF}
  NewElement^.Next := NIL;

  IF Tail = NIL THEN
    Head := NewElement
  ELSE
    Tail^.Next := NewElement;

  Tail := NewElement;
  Inc(Count);
END;

FUNCTION TListe.Get(Index: Integer; {$IFDEF DOS} Data: Pointer {$ELSE} VAR Data: T {$ENDIF}): BOOLEAN;
VAR
  Current: PListeElement;
  CountIndex: Integer;
BEGIN
  Current := Head;
  CountIndex := 0;
  WHILE (Current <> NIL) AND (CountIndex < Index) DO BEGIN
    Current := Current^.Next;
    Inc(CountIndex);
  END;

  IF Current = NIL THEN
    Get := FALSE
  ELSE BEGIN
    {$IFDEF DOS}
      Move(Current^.Data^, Data^, DataSize);
    {$ELSE}
      Data := Current^.Data;
    {$ENDIF}
    Get := TRUE;
  END;
END;

FUNCTION TListe.Remove(Index: Integer): BOOLEAN;
VAR
  Current, Prev: PListeElement;
  CountIndex: Integer;
BEGIN
  Remove := FALSE;
  IF Head = NIL THEN EXIT;

  Current := Head;
  Prev := NIL;
  CountIndex := 0;

  WHILE (Current <> NIL) AND (CountIndex < Index) DO BEGIN
    Prev := Current;
    Current := Current^.Next;
    Inc(CountIndex);
  END;

  IF Current = NIL THEN EXIT;

  IF Prev = NIL THEN
    Head := Current^.Next
  ELSE
    Prev^.Next := Current^.Next;

  IF Current = Tail THEN
    Tail := Prev;

  {$IFDEF DOS} FreeMem(Current^.Data, DataSize); {$ENDIF}
  Dispose(Current);
  Dec(Count);
  Remove := TRUE;
END;

FUNCTION TListe.IsEmpty: BOOLEAN;
BEGIN
  IsEmpty := Head = NIL;
END;

PROCEDURE TListe.Clear;
VAR
  TempElement: PListeElement;
BEGIN
  WHILE Head <> NIL DO BEGIN
    TempElement := Head;
    Head := Head^.Next;
    {$IFDEF DOS} FreeMem(TempElement^.Data, DataSize); {$ENDIF}
    Dispose(TempElement);
  END;
  Tail := NIL;
  Count := 0;
END;

FUNCTION TListe.Anzahl: Integer;
BEGIN
  Anzahl := Count;
END;

FUNCTION TListe.Start: PListeElement;
BEGIN
  Start := Head;
END;

BEGIN
END.

