UNIT Stack;

{$I OsWahl.inc}

{$IFNDEF DOS} {$MODE OBJFPC} {$H+} {$ENDIF}

INTERFACE

TYPE

{$IFDEF DOS}

  PStackElement = ^TStackElement;
  TStackElement = RECORD
    Data: Pointer;
    Next: PStackElement;
  END;

  TStack = OBJECT
    Private
      Top: PStackElement;
      DataSize: Integer;

{$ELSE}

  GENERIC TStack<T> = OBJECT
    Private
      TYPE
        PStackElement = ^TStackElement;
        TStackElement = RECORD
          Data: T;
          Next: PStackElement;
        END;
      VAR
        Top: PStackElement;

{$ENDIF}

    Public
      PROCEDURE Init {$IFDEF DOS} (SizeOfData: Integer) {$ENDIF} ;
      PROCEDURE Push(Data: {$IFDEF DOS} Pointer {$ELSE} T {$ENDIF});
      FUNCTION Pop({$IFDEF DOS} Data: Pointer {$ELSE} VAR Data: T {$ENDIF}): BOOLEAN;
      FUNCTION IsEmpty: BOOLEAN;
      PROCEDURE Clear;
    END;

IMPLEMENTATION

{$IFDEF DOS} USES DOS; {$ENDIF}

PROCEDURE TStack.Init {$IFDEF DOS} (SizeOfData: Integer) {$ENDIF} ;
BEGIN
  Top := NIL;
  {$IFDEF DOS} DataSize := SizeOfData; {$ENDIF}
END;

PROCEDURE TStack.Push(Data: {$IFDEF DOS} Pointer {$ELSE} T {$ENDIF});
VAR
  NewElement: PStackElement;
BEGIN
  New(NewElement);
  {$IFDEF DOS}
    GetMem(NewElement^.Data, DataSize);
    Move(Data^, NewElement^.Data^, DataSize);
  {$ELSE}
    NewElement^.Data := Data;
  {$ENDIF}
  NewElement^.Next := Top;
  Top := NewElement;
END;

FUNCTION TStack.Pop({$IFDEF DOS} Data: Pointer {$ELSE} VAR Data: T {$ENDIF}): BOOLEAN;
VAR
  TempElement: PStackElement;
BEGIN
  IF Top = NIL THEN
    Pop := FALSE
  ELSE BEGIN
    {$IFDEF DOS}
      Move(Top^.Data^, Data^, DataSize);
    {$ELSE}
      Data := Top^.Data;
    {$ENDIF}
    TempElement := Top;
    Top := Top^.Next;
    {$IFDEF DOS} FreeMem(TempElement^.Data, DataSize); {$ENDIF}
    Dispose(TempElement);
    Pop := TRUE;
  END;
END;

FUNCTION TStack.IsEmpty: BOOLEAN;
BEGIN
  IsEmpty := Top = NIL;
END;

PROCEDURE TStack.Clear;
VAR
  TempElement: PStackElement;
BEGIN
  WHILE Top <> NIL DO BEGIN
    TempElement := Top;
    Top := Top^.Next;
    {$IFDEF DOS} FreeMem(TempElement^.Data, DataSize); {$ENDIF}
    Dispose(TempElement);
  END;
END;

BEGIN
END.
