PROGRAM freiburg;

{$I OsWahl.inc}

USES Stdio, Stack, Liste;

TYPE
   PEintrag = ^TEintrag;
   TEintrag = RECORD
      M1, M2, M3, N1, N2, L1: INTEGER;
      PROG, BEZEICHNER, CODE: STRING[50];
   END;

   TMEintrag = {$IFDEF DOS} PEintrag {$ELSE} specialize TListe<TEintrag>.PListeElement {$ENDIF};
   TMListe   = {$IFDEF DOS} TListe {$ELSE} specialize TListe<TEintrag> {$ENDIF};

   TMenueManager = OBJECT
      Gesamt, Auswahl: TMListe;
      aktuell: TEintrag;
      MenueStapel: {$IFDEF DOS} TStack {$ELSE} specialize TStack<TEintrag> {$ENDIF};
      MenueAngezeigt: BOOLEAN;
      PROCEDURE Init;
      PROCEDURE Freigeben;
      PROCEDURE LoadCSV(CONST FileName: STRING);
      PROCEDURE Zeige_Menue;
      FUNCTION  Bearbeite_Auswahl(wahl: CHAR): BOOLEAN;
      FUNCTION  Finde_Menuepunkt(M1, M2, M3: INTEGER; VAR Ergebnis: TEintrag): BOOLEAN;
      FUNCTION  Hat_Nachfolge_Menue(N1, N2: INTEGER): BOOLEAN;
      FUNCTION  Aktuelle_Menuepunkte(VAR Ergebnis: TMListe): BOOLEAN;
      PROCEDURE Fuehre_Menue_Aus;
   END;

PROCEDURE Init_Eintrag(VAR eintrag: TEintrag);
BEGIN
   WITH Eintrag DO BEGIN
      M1 := 0; M2 := 0; M3 := 0; N1 := 0; N2 := 0; L1 := 0;
      PROG := ''; CODE := ''; BEZEICHNER := '';
   END;
END;

PROCEDURE TMenueManager.Init;
BEGIN
   Gesamt.Init {$IFDEF DOS} (SizeOf(TEintrag)) {$ENDIF};;
   Auswahl.Init {$IFDEF DOS} (SizeOf(TEintrag)) {$ENDIF};;
   Init_Eintrag(aktuell); aktuell.BEZEICHNER := 'Hauptmenue';
   MenueAngezeigt := False;
   MenueStapel.Init {$IFDEF DOS} (SizeOf(TEintrag)) {$ENDIF};
END;

PROCEDURE TMenueManager.Freigeben;
BEGIN
   Gesamt.Clear; Auswahl.Clear;
   MenueStapel.Clear;
END;

FUNCTION TMenueManager.Finde_Menuepunkt(M1, M2, M3: INTEGER; VAR Ergebnis: TEintrag): BOOLEAN;
VAR Zeiger: TMEintrag;
    wert: BOOLEAN;
BEGIN
   wert := False;
   Zeiger := Gesamt.Head;
   WHILE (Zeiger <> NIL) AND NOT wert DO BEGIN
      IF (Zeiger^.Data.M1 = M1) AND (Zeiger^.Data.M2 = M2) AND (Zeiger^.Data.M3 = M3) THEN BEGIN
         Ergebnis := Zeiger^.Data; wert := True;
      END;
      Zeiger := Zeiger^.Next;
   END;
   Finde_Menuepunkt := wert;
END;

FUNCTION TMenueManager.Hat_Nachfolge_Menue(N1, N2: INTEGER): BOOLEAN;
VAR Zeiger: TMEintrag;
   wert: BOOLEAN;
BEGIN
   wert := False;
   Zeiger := Gesamt.Head;
   WHILE (Zeiger <> NIL) AND NOT wert DO BEGIN
      IF (Zeiger^.Data.M1 = N1) AND (Zeiger^.Data.M2 = N2)
         THEN wert := True;
      Zeiger := Zeiger^.Next;
   END;
   Hat_Nachfolge_Menue := wert;
END;

FUNCTION TMenueManager.Aktuelle_Menuepunkte(VAR Ergebnis: TMListe): BOOLEAN;
VAR Zeiger: TMEintrag;
BEGIN
   Ergebnis.Clear;
   Zeiger := Gesamt.Head;
   WHILE Zeiger <> NIL DO BEGIN
      IF (Zeiger^.Data.M1 = aktuell.M1) AND (Zeiger^.Data.M2 = aktuell.M2) THEN
         Ergebnis.Add(Zeiger^.Data);
      Zeiger := Zeiger^.Next;
   END;
   Aktuelle_Menuepunkte := Ergebnis.Anzahl <> 0;
END;

PROCEDURE TMenueManager.Zeige_Menue;
VAR Zeiger: TMEintrag;
    i: INTEGER;
BEGIN
   IF NOT Aktuelle_Menuepunkte(Auswahl) THEN
      Writeln('Fehler: Menuezeilen nicht vorhanden.')
   ELSE BEGIN
      Writeln(aktuell.BEZEICHNER, ':');
      Zeiger := Auswahl.Head;
      WHILE Zeiger <> NIL DO BEGIN
         Writeln(Zeiger^.Data.M3, ': ', Zeiger^.Data.BEZEICHNER);
         FOR i := 1 TO Zeiger^.Data.L1 DO Writeln;
         Zeiger := Zeiger^.Next;
      END;
      Writeln('0: Zurueck');
   END;
END;

FUNCTION TMenueManager.Bearbeite_Auswahl(wahl: CHAR): BOOLEAN;
VAR Punkt: TEintrag;
    ziffer,w: INTEGER;
BEGIN
   Bearbeite_Auswahl := False;
   IF wahl in ['0'..'9'] THEN VAL(wahl, ziffer, w)
   ELSE BEGIN
      Writeln('Ungueltige Auswahl.');
      Exit;
   END;
   IF ziffer = 0 THEN BEGIN
      MenueAngezeigt := False;
      IF MenueStapel.IsEmpty THEN BEGIN
         Writeln('Programmende.');
         Bearbeite_Auswahl := True; Exit;
      END;
      MenueStapel.Pop({$IFDEF DOS}@{$ENDIF}aktuell);
      Exit;
   END;
   IF NOT Finde_Menuepunkt(aktuell.M1, aktuell.M2, ziffer, Punkt) THEN BEGIN
      Writeln('Ungueltige Auswahl.');
      Exit;
   END;
   aktuell.BEZEICHNER := Punkt.BEZEICHNER;
   CASE UpCase(Punkt.PROG[1]) OF
     'P': Writeln('Starte Programm: ', Punkt.CODE);
     'M': IF Hat_Nachfolge_Menue(Punkt.N1, Punkt.N2) THEN BEGIN
             MenueStapel.Push({$IFDEF DOS}@{$ENDIF}aktuell);
             aktuell.M1 := Punkt.N1; aktuell.M2 := Punkt.N2;
             MenueAngezeigt := False;
          END
          ELSE Writeln('Ungueltige Auswahl. Aufgerufenes Menue nicht gefunden.');
      ELSE Writeln('Fehler: Falscher Programmcode.');
   END;
END;

PROCEDURE TMenueManager.Fuehre_Menue_Aus;
VAR wahl: CHAR;
    ende: BOOLEAN;
BEGIN
   ende := False;
   REPEAT
      IF NOT MenueAngezeigt THEN BEGIN
         Zeige_Menue; MenueAngezeigt := True;
      END;
      Write('Auswahl: '); ReadLn(wahl);
      ende := Bearbeite_Auswahl(wahl);
   UNTIL ende;
END;

PROCEDURE TMenueManager.LoadCSV(CONST FileName: STRING);
VAR
   CSVFile: TEXT;
   Temp: TEintrag;
   Zeile: STRING;
   FUNCTION Strip: STRING;
   VAR k: INTEGER;
   BEGIN
      k := Pos(';', Zeile); IF k = 0 THEN k := LENGTH(Zeile) + 1;
      Strip := Copy(Zeile, 1, k - 1);
      Zeile := Copy(Zeile, k + 1, 255);
   END;
BEGIN
   Assign(CSVFile, FileName); Reset(CSVFile);
   ReadLn(CSVFile, Zeile);
   WHILE NOT EOF(CSVFile) DO BEGIN
      ReadLn(CSVFile, Zeile);
      WITH Temp DO BEGIN
         M1 := Str2Int(Strip); M2 := Str2Int(Strip); M3 := Str2Int(Strip);
         PROG := Strip; BEZEICHNER := Strip; CODE := Strip;
         N1 := Str2Int(Strip); N2 := Str2Int(Strip); L1 := Str2Int(Strip);
      END;
      IF Temp.PROG <> 'D' THEN
         Gesamt.Add(Temp);
   END;
   Close(CSVFile);
END;

VAR MenueManager: TMenueManager;

BEGIN
   WITH Menuemanager DO BEGIN
      Init; LoadCSV('daten.csv');
      Fuehre_Menue_Aus;
      Freigeben;
   END;
   Writeln('Beenden'); Taste0;
END.

