import csv
import logging

logging.basicConfig(level=logging.INFO, format='%(levelname)s - %(message)s')

class MenueZeile:
    def __init__(self, m1, m2, m3, prog, bezeichner, code, n1, n2, l1):
        self.m1 = m1 ; self.m2 = m2 ; self.m3 = m3
        self.prog = prog ; self.bezeichner = bezeichner ; self.code = code
        self.n1 = n1 ; self.n2 = n2 ; self.l1 = l1

class MenueManager:
    def __init__(self, dateiname):
        self.daten = self.csv_lesen(dateiname)
        self.m1 = self.m2 = 0
        self.menue_stapel = []  
        self.bezeichnung = "Hauptmenü"
        self.menue_angezeigt = False

    def aktuelle_menuepunkte(self):
        return [zeile for zeile in self.daten if zeile.m1 == self.m1 and zeile.m2 == self.m2]

    def finde_menuepunkt(self, auswahl):
        return next((zeile for zeile in self.daten if zeile.m1 == self.m1 and zeile.m2 == self.m2 and zeile.m3 == auswahl), None)

    def zeige_menue(self):
        menuepunkte = self.aktuelle_menuepunkte()
        if not menuepunkte:
            logging.error("Menüzeilen nicht vorhanden.")
        else:
            print(f"\n{self.bezeichnung}:")
            for punkt in menuepunkte:
                print(f"{punkt.m3}: {punkt.bezeichner}")
                for i in range(punkt.l1):
                    print()
            print("\n0: Zurück")

    def hat_nachfolge_menue(self, n1, n2):
        return any(zeile for zeile in self.daten if zeile.m1 == n1 and zeile.m2 == n2)

    def bearbeite_auswahl(self, auswahl):
        if auswahl == 0:
            self.menue_angezeigt = False
            if self.menue_stapel:
                self.m1, self.m2, self.bezeichnung = self.menue_stapel.pop()
                return False
            else:
                logging.info("Programmende.")
                return True

        punkt = self.finde_menuepunkt(auswahl)
        if not punkt:
            logging.warning("Ungültige Auswahl. Keine Einträge für das gewählte Menü.")
            return False
            
        self.bezeichnung = punkt.bezeichner
        if punkt.prog.upper() == 'P':
            logging.info(f"Starte Programm: {punkt.code}")
  
        elif punkt.prog.upper() == 'M':
            if self.hat_nachfolge_menue(punkt.n1, punkt.n2):
                self.menue_stapel.append((self.m1, self.m2, self.bezeichnung))
                self.m1, self.m2 = punkt.n1, punkt.n2
                self.menue_angezeigt = False
            else:
                logging.warning("Ungültige Auswahl. Aufgerufenes Menü nicht gefunden.")
        else:
            logging.error("Falscher Programmcode.")
        
        return False

    def fuehre_menue_aus(self):
        ende = False
        while not ende:
            if not self.menue_angezeigt:
                self.zeige_menue()
                self.menue_angezeigt = True
            try:
                auswahl = int(input("Auswahl: "))
                ende = self.bearbeite_auswahl(auswahl)
            except ValueError:
                logging.error("Ungültige Eingabe. Bitte geben Sie eine Zahl ein.")

    def csv_lesen(self, dateiname):
        def str2int(string):
            try:
                return int(string)
            except ValueError:
                return -1
        daten = []
        with open(dateiname, newline='') as csvfile:
            for zeile in csv.DictReader(csvfile, delimiter=';'):
                if zeile['PROG'] != 'D':
                    daten.append(MenueZeile(
                        str2int(zeile['M1']), str2int(zeile['M2']), str2int(zeile['M3']),
                        zeile['PROG'], zeile['BEZEICHNER'], zeile['CODE'],
                        str2int(zeile['N1']), str2int(zeile['N2']), str2int(zeile['L1'])
                    ))
        return daten

def main():
    menue_manager = MenueManager('freiburg.csv')
    menue_manager.fuehre_menue_aus()

if __name__ == "__main__":
    main()
